// This software was written by Gregory L Montgomery
// and is completely covered by the 
// GPL General Public License

// Date of release 1/7/1998 

// Programmer Gregory Montgomery

// This is the .C file for the AD  CLASS


#include <ctype.h>
#include <string.h>
#include "thermister.h"
#include "Gen.h"

// This is an abstract class

// ----------------------------------------------------------- */
// constructors                                                */

// Default thermister

Generic::Generic() {

     // strcpy(ID,"Generic");
    // The listed values are the values
   // for the thermister from your local 
  // Radio Shack Store.
 // Part Number 271-110 
// 10k ohm +- 1% at 25 Degrees C

    Range = new double[35];
    Range[0]  =    757.9;
    Range[1]  =    857.5;
    Range[2]  =    973.5;
    Range[3]  =   1108.0;
    Range[4]  =   1366.0;
    Range[5]  =   1451.0;
    Range[6]  =   1669.0;
    Range[7]  =   1924.0;
    Range[8]  =   2229.0;
    Range[9]  =   2589.0;
    Range[10] =   3021.0;
    Range[11] =   3537.0;
    Range[12] =   4161.0;
    Range[13] =   4912.0;
    Range[14] =   5828.0;
    Range[15] =   6941.0;
    Range[16] =   8313.0;
    Range[17] =  10000.0;
    Range[18] =  12090.0;
    Range[19] =  14680.0;
    Range[20] =  17960.0;
    Range[21] =  22050.0;
    Range[22] =  27280.0;
    Range[23] =  33890.0;
    Range[24] =  42450.0;
    Range[25] =  53390.0;
    Range[26] =  67740.0;
    Range[27] =  86390.0;
    Range[28] = 111300.0;
    Range[29] = 144000.0;
    Range[30] = 188400.0;
    Range[31] = 247500.0;
    Range[32] = 329200.0;
    Range[33] = -1.0;
    Range[34] = -1.0;

    Temps = new double[35];
    Temps[0]  =    110.0;
    Temps[1]  =    105.0;
    Temps[2]  =    100.0;
    Temps[3]  =     95.0;
    Temps[4]  =     90.0;
    Temps[5]  =     85.0;
    Temps[6]  =     80.0;
    Temps[7]  =     75.0;
    Temps[8]  =     70.0;
    Temps[9]  =     65.0;
    Temps[10] =     60.0;
    Temps[11] =     55.0;
    Temps[12] =     50.0;
    Temps[13] =     45.0;
    Temps[14] =     40.0;
    Temps[15] =     35.0;
    Temps[16] =     30.0;
    Temps[17] =     25.0;
    Temps[18] =     20.0;
    Temps[19] =     15.0;
    Temps[20] =     10.0;
    Temps[21] =      5.0;
    Temps[22] =      0.0;
    Temps[23] =     -5.0;
    Temps[24] =    -10.0;
    Temps[25] =    -15.0;
    Temps[26] =    -20.0;
    Temps[27] =    -25.0;
    Temps[28] =    -30.0;
    Temps[29] =    -35.0;
    Temps[30] =    -40.0;
    Temps[31] =    -45.0;
    Temps[32] =    -55.0;
    Temps[33] =     -1.0;
    Temps[34] =     -1.0;
    deltaT = 5.0;
    Vref = 5.03;
    Rref = 10000.0;
}


//-----------------------------------------------------------/
// destructor Virtual : Every class that inherits from this will have to 
// declare a destructor like it or not. 

Generic::~Generic(){
  delete []Range;
}

//-----------------------------------------------------------
// This is a basic copy constructor

thermist * Generic::clone()const{
return new Generic(*this);
}


//--------------------------------------------------------------
// A virtual kind function

thermist::Types Generic::kind() const {
    return ((thermist::Types) Gen);
}

//----------------------------------------------------------------/
// This will generically allow us to ID which type of AD
// we are dealing with

char * Generic::Ident()const {
    return "Generic";
} 

//---------------------------------------------------------
// This will handle the calibration for Generic For Resister;

void   Generic::CalibrateR(const double Hm){Rref = Hm;}

//---------------------------------------------------------
// This will handle the calibration for Generic For Voltage;

void   Generic::CalibrateV(const double Hm){Vref += Hm;}

//--------------------------------------------------------
// This is the Fuction that dose the convertion for us

double Generic::VoltsToTemp(double const Voltage){

    int i;
    double deltaR,M,R2;

    i = 0;
    if(Voltage != 0)
    	R2 = ((Rref * Vref)/Voltage) - Rref;
    else
        R2 = 3292000;
    while((Range[i] < R2)&&(i < 33))
         i++;

//cerr << "Range[" << i << "] = " << Range[i] << "  ";
//cerr << "Range[" << i-1 << "] = " << Range[i-1] << '\n';
//cerr << "R2 " << R2 << '\n';
    if(Range[i] == R2) return Temps[i];

    deltaR = (Range[i] - R2)/(Range[i] - Range[i-1]); 
    M = deltaR*5.0 + Temps[i]; 
//cerr << "M " << M << '\n';
    return deltaT*deltaR + Temps[i];
}


